# Client API (version 1) <br/> Passwords Microservices Client SDK for Node.js

Node.js client API for Passwords microservice is a thin layer on the top of
communication protocols. It hides details related to specific protocol implementation
and provides high-level API to access the microservice for simple and productive development.

* [Installation](#install)
* [Getting started](#get_started)
* [IPasswordsClientV1 interface](#interface)
    - [setPassword()](#operation1)
    - [deletePassword()](#operation2)
    - [authenticate()](#operation3)
    - [changePassword()](#operation4)
    - [resetPassword()](#operation5)
    - [recoverPassword()](#operation6)
* [PasswordsHttpClientV1 class](#client_rest)
* [PasswordsSenecaClientV1 class](#client_seneca)
* [PasswordsDirectClientV1 class](#client_direct)
* [PasswordsNullClientV1 class](#client_null)

## <a name="install"></a> Installation

To work with the client SDK add dependency into package.json file:

```javascript
{
    ...
    "dependencies": {
        ....
        "pip-clients-passwords-node": "^1.0.*",
        ...
    }
}
```

Then download the dependency using **npm**:

```javascript
# Installing dependencies
npm install

# Updating dependencies
npm update
```

## <a name="get_started"></a> Getting started

This is a simple example on how to work with the microservice using REST client:

```javascript
// Get Client SDK for Version 1 
var sdk = new require('pip-clients-passwords-node');

// Client configuration
var config = {
    connection: {
        protocol: 'http',
        host: 'localhost', 
        port: 8080
    }
};

// Create the client instance
var client = sdk.PasswordsHttpClientV1(config);

// Open client connection to the microservice
client.open(null, function(err) {
    if (err) {
        console.error(err);
        return; 
    }
    
    console.log('Opened connection');
        
    // Set password for a new user
    client.setPassword(
        '123',
        'test123',
        function (err) {
            if (err) {
                console.error(err);
                return;
            }
            
            // Authenticate user
            client.authenticate(
                '123',
                'test123',
                function(err, autnenticated) {
                    if (err) {
                        console.error(err);
                        return;
                    }
                    
                    console.log('Authentication result is');
                    console.log(authentication);
                    
                    // Close connection
                    client.close(null); 
                }
            );
        }
    );
});
```

## <a name="interface"></a> IPasswordsClientV1 interface

If you are using Typescript, you can use IPasswordsClient as a common interface across all client implementations. 
If you are using plain Javascript, you shall not worry about IPasswordsClient interface. You can just expect that
all methods defined in this interface are implemented by all client classes.

```javascript
interface IPasswordsClientV1 {
    setPassword(correlationId, userId, password, callback);
    deletePassword(correlationId, userId, callback);
    authenticate(correlationId, userId, password, callback);
    changePassword(correlationId, userId, oldPassword, newPassword, callback);
    resetPassword(correlationId, userId, code, password, callback);
    recoverPassword(correlationId, userId, callback);
}
```

### <a name="operation1"></a> setPassword(correlationId, userId, password, callback)

Sets password for a new user in the system and creates an object for him.

**Arguments:** 
- correlationId: string - id that uniquely identifies transaction
- userId: string - unique user id generated by the client
- password: string - user password
- callback: (err, userPassword) => void - callback function
  - err: Error - occured error or null for success
 
### <a name="operation2"></a> deletePassword(correlationId, userId, callback)

Deletes user password from the system (use it carefully!)

**Arguments:** 
- correlationId: string - id that uniquely identifies transaction
- userId: string - unique user id
- callback: (err) => void - callback function
  - err: Error - occured error or null for success

### <a name="operation3"></a> authenticate(correlationId, userId, password, callback)

Authenticates user using ID/password combination and returns user object.

**Arguments:** 
- correlationId: string - id that uniquely identifies transaction
- userId: string - unique user id to identify the user
- password: string - user password
- callback: (err, userPassword) => void - callback function
  - err: Error - occured error or null for success
  - authenticated: boolean - True when authentication was successful
 
### <a name="operation4"></a> changePassword(correlationId, userId, oldPassword, newPassword, callback)

Changes user password by providing old password

**Arguments:** 
- correlationId: string - id that uniquely identifies transaction
- userId: string - unique user id to identify the user
- oldPassword: string - old user password
- newPassword: string - new user password
- callback: (err, userPassword) => void - callback function
  - err: Error - occured error or null for success

### <a name="operation5"></a> resetPassword(correlationId, userId, code, password, callback)

Resets user password by providing recovery code

**Arguments:** 
- correlationId: string - id that uniquely identifies transaction
- userId: string - unique user id to identify the user
- code: string - password recovery code
- password: string - new user password
- callback: (err, userPassword) => void - callback function
  - err: Error - occured error or null for success

### <a name="operation6"></a> recoverPassword(correlationId, userId, callback)

Generates password recovery code for the user and sends it via email

**Arguments:** 
- correlationId: string - id that uniquely identifies transaction
- userId: string - unique user id to identify the user
- callback: (err, code) => void - callback function
  - err: Error - occured error or null for success

## <a name="client_rest"></a> PasswordsHttpClientV1 class

PasswordsHttpClient is a client that implements HTTP protocol

```javascript
class PasswordsHttpClientV1 extends CommandableHttpClient implements IPasswordsClient {
    constructor(config: any);
    setReferences(references);
    open(correlationId, callback);
    close(correlationId, callback);
    setPassword(correlationId, userId, password, callback);
    deletePassword(correlationId, userId, callback);
    authenticate(correlationId, userId, password, callback);
    changePassword(correlationId, userId, oldPassword, newPassword, callback);
    resetPassword(correlationId, userId, code, password, callback);
    recoverPassword(correlationId, userId, callback);
}
```

**Constructor config properties:** 
- connection: object - HTTP transport configuration options
  - protocol: string - HTTP protocol - 'http' or 'https' (default is 'http')
  - host: string - IP address/hostname binding (default is '0.0.0.0')
  - port: number - HTTP port number

## <a name="client_seneca"></a> PasswordsSenecaClientV1 class

PasswordsSenecaClientV1 is a client that implements Seneca protocol

```javascript
class PasswordsSenecaClientV1 extends CommandableSenecaClient implements IPasswordsClientV1 {
    constructor(config: any);        
    setReferences(references);
    open(correlationId, callback);
    close(correlationId, callback);
    setPassword(correlationId, userId, password, callback);
    deletePassword(correlationId, userId, callback);
    authenticate(correlationId, userId, password, callback);
    changePassword(correlationId, userId, oldPassword, newPassword, callback);
    resetPassword(correlationId, userId, code, password, callback);
    recoverPassword(correlationId, userId, callback);
}
```

**Constructor config properties:** 
- connection: object - (optional) Seneca transport configuration options. See http://senecajs.org/api/ for details.
  - protocol: string - Seneca transport type 
  - host: string - IP address/hostname binding (default is '0.0.0.0')
  - port: number - Seneca port number

## <a name="client_direct"></a> PasswordsDirectClientV1 class

PasswordsDirectClientV1 is a null client for testing. It never fails and always returns a fake UserPassword object

```javascript
class PasswordsDirectClientV1 extends DirectClient implements IPasswordsClientV1 {
    constructor(config: any);        
    setReferences(references);
    open(correlationId, callback);
    close(correlationId, callback);
    setPassword(correlationId, userId, password, callback);
    deletePassword(correlationId, userId, callback);
    authenticate(correlationId, userId, password, callback);
    changePassword(correlationId, userId, oldPassword, newPassword, callback);
    resetPassword(correlationId, userId, code, password, callback);
    recoverPassword(correlationId, userId, callback);
```

## <a name="client_null"></a> PasswordsNullClientV1 class

PasswordsNullClientV1 is a null client for testing. It never fails and always returns a fake UserPassword object

```javascript
class PasswordsNullClientV1 implements IPasswordsClientV1 {
    setPassword(correlationId, userId, password, callback);
    deletePassword(correlationId, userId, callback);
    authenticate(correlationId, userId, password, callback);
    changePassword(correlationId, userId, oldPassword, newPassword, callback);
    resetPassword(correlationId, userId, code, password, callback);
    recoverPassword(correlationId, userId, callback);
```
